import {
  Endpoint,
  RPCError,
  RPC_CODE_TO_HTTP_STATUS_CODE
} from "../chunk-YUHQMGGG.js";

// src/adapters/fastify.ts
import fp from "fastify-plugin";
var rpcFastify = fp((fastify, opts, done) => {
  const { prefix, transformer, router } = opts;
  fastify.setValidatorCompiler(createValidatorCompiler(transformer));
  fastify.setSerializerCompiler(createSerializerCompiler(transformer));
  function registerRoutes(key = "", route) {
    if (route instanceof Endpoint) {
      let attachCtx2 = function(req, res) {
        if (!req._httpRpcCtx)
          req._httpRpcCtx = { req, res, input: req.body ?? req.query };
        return req._httpRpcCtx;
      };
      var attachCtx = attachCtx2;
      const schema = {
        response: {
          200: route.output
        }
      };
      if (route.method === "GET") {
        Object.assign(schema, {
          querystring: route.input
        });
      } else if (route.method === "POST") {
        Object.assign(schema, {
          body: route.input
        });
      }
      const handler = async (req, res) => {
        const ctx = attachCtx2(req, res);
        const data = await route.handler(ctx);
        if (res.sent)
          return;
        return { data };
      };
      const preHandlerWrapper = (cb) => {
        return async (req, res) => {
          const ctx = attachCtx2(req, res);
          const result = await cb(ctx);
          Object.assign(ctx, result);
        };
      };
      fastify.route({
        method: route.method,
        url: `${prefix}${key}`,
        // @ts-ignore
        preHandler: route.middlewares.map(preHandlerWrapper),
        // @ts-ignore
        handler,
        schema
      });
    } else {
      for (const k in route) {
        const value = route[k];
        registerRoutes(`${key}/${k}`, value);
      }
    }
  }
  registerRoutes("", router);
  fastify.setErrorHandler(async (err, req, res) => {
    res.header("Content-Type", "application/problem+json");
    let result = {
      status: 500,
      title: err.message ?? "Internal Server Error",
      code: "INTERNAL_SERVER_ERROR",
      instance: req.routeOptions.url
    };
    if (err instanceof RequestValidationError) {
      const { statusCode = 400, code, errors } = err;
      Object.assign(result, {
        status: statusCode,
        title: "Bad Request",
        detail: "Request validation failed",
        code,
        errors
      });
    } else if (err instanceof RPCError) {
      const { type, title, detail, code, extensions } = err;
      const httpStatusCode = RPC_CODE_TO_HTTP_STATUS_CODE[code] ?? 500;
      Object.assign(result, {
        status: httpStatusCode,
        type,
        title,
        detail,
        code
      });
      Object.assign(result, extensions);
    }
    const onErrResult = async () => {
      try {
        return await opts.onError?.(result, err);
      } catch (err2) {
        console.error(err2);
      }
    };
    result = await onErrResult() ?? result;
    return res.status(result.status ?? 500).send(JSON.stringify(result));
  });
  done();
});
var RequestValidationError = class extends Error {
  errors;
  constructor(err) {
    super("Request doesn't match the schema");
    this.name = "RequestValidationError";
    this.errors = err.errors;
  }
};
var ResponseValidationError = class extends Error {
  errors;
  constructor(error) {
    super("Response doesn't match the schema");
    this.name = "ResponseValidationError";
    this.errors = error.errors;
  }
};
var createValidatorCompiler = (transformer) => {
  const validatorCompiler = ({ schema, method }) => {
    return (data) => {
      if (method === "GET") {
        data = JSON.parse(data.input ?? "{}");
      }
      if (transformer) {
        data = transformer.deserialize(data);
      }
      try {
        return { value: schema.parse(data) };
      } catch (err) {
        return { error: new RequestValidationError(err) };
      }
    };
  };
  return validatorCompiler;
};
var createSerializerCompiler = (transformer) => {
  const serializerCompiler = ({ schema }) => ({ data }) => {
    const parsedResult = schema.safeParse(data);
    if (parsedResult.success) {
      let output = parsedResult.data;
      if (transformer) {
        output = transformer.serialize(output);
      }
      return JSON.stringify({ data: output });
    }
    throw new ResponseValidationError(parsedResult.error);
  };
  return serializerCompiler;
};
export {
  rpcFastify
};
