// src/route.ts
import z from "zod";
var Endpoint = class {
  method;
  input;
  output;
  handler;
  middlewares;
  constructor(input) {
    this.method = input.method;
    this.input = input?.input;
    this.output = input.output;
    this.handler = input.handler ?? (() => {
      throw new Error("Handler not implemented");
    });
    this.middlewares = input.middlewares;
  }
};
var Route = class _Route {
  #input;
  #output;
  #middlewares;
  #afterMiddlewares;
  constructor(data) {
    this.#input = data?.input ?? z.any();
    this.#output = data?.output ?? z.any();
    this.#middlewares = data?.middlewares ?? [];
    this.#afterMiddlewares = data?.afterMiddlewares ?? [];
  }
  #isZodObject(schema) {
    return schema?._def?.typeName === "ZodObject";
  }
  #isZodAny(schema) {
    return schema?._def?.typeName === "ZodAny";
  }
  #prepare(data) {
    let input = this.#input;
    if (this.#isZodAny(input)) {
      input = data.input;
    } else if (this.#isZodObject(input) && this.#isZodObject(data.input)) {
      input = input.merge(data.input);
    }
    const output = data.output ?? this.#output;
    return new _Route({
      input,
      output,
      method: data.method,
      handler: data.handler,
      middlewares: this.#middlewares.concat(data.middleware).filter(Boolean),
      afterMiddlewares: this.#afterMiddlewares.slice()
    });
  }
  input(schema) {
    return this.#prepare({ input: schema });
  }
  output(schema) {
    return this.#prepare({ output: schema });
  }
  middleware(middleware) {
    return this.#prepare({ middleware });
  }
  post(cb) {
    return new Endpoint({
      method: "POST",
      input: this.#input,
      output: this.#output,
      handler: cb,
      middlewares: this.#middlewares.concat(this.#afterMiddlewares)
    });
  }
  get(cb) {
    return new Endpoint({
      method: "GET",
      input: this.#input,
      output: this.#output,
      handler: cb,
      middlewares: this.#middlewares.concat(this.#afterMiddlewares)
    });
  }
  on(event, cb) {
    switch (event) {
      case "afterMiddlewares":
        this.#afterMiddlewares.push(cb);
        break;
    }
  }
};
function createRoute() {
  return new Route();
}

// src/error.ts
var RPC_CODE_TO_HTTP_STATUS_CODE = {
  PARSE_ERROR: 400,
  BAD_REQUEST: 400,
  UNAUTHORIZED: 401,
  NOT_FOUND: 404,
  FORBIDDEN: 403,
  METHOD_NOT_SUPPORTED: 405,
  TIMEOUT: 408,
  CONFLICT: 409,
  PRECONDITION_FAILED: 412,
  PAYLOAD_TOO_LARGE: 413,
  UNSUPPORTED_MEDIA_TYPE: 415,
  UNPROCESSABLE_CONTENT: 422,
  TOO_MANY_REQUESTS: 429,
  CLIENT_CLOSED_REQUEST: 499,
  INTERNAL_SERVER_ERROR: 500,
  NOT_IMPLEMENTED: 501
};
var RPCError = class extends Error {
  code;
  type;
  title;
  detail;
  extensions;
  constructor(opts) {
    const title = opts.title ?? opts.code;
    super(title);
    this.name = "RPCError";
    this.code = opts.code;
    this.type = opts.type;
    this.title = title;
    this.detail = opts.detail;
    this.extensions = opts.extensions;
  }
};

export {
  Endpoint,
  createRoute,
  RPC_CODE_TO_HTTP_STATUS_CODE,
  RPCError
};
